{% extends "base.html.jinja" %}
{% block title %}All Symbols{% endblock %}
{% block page_header %}<h1>All Symbols</h1>{% endblock %}
{% block content %}

    {% import 'lists.html.jinja' as lists with context %}
    {{ lists.symbols(all_functions, all_variables) }}

{% endblock %}
