{% extends "base.html.jinja" %}
{% import 'lists.html.jinja' as lists with context %}
{% block title %}Folder{% endblock %}
{% block content %}
    {{ lists.breadcrumbs(folder) }}
    {{ lists.directory(folder.sub_folders, folder.files) }}
{% endblock %}