{% extends "base.html.jinja" %}
{% import 'lists.html.jinja' as lists with context %}
{% block title %}Single File{% endblock %}
{% block content %}
    {{ lists.breadcrumbs(file.folder, file) }}
    {{ lists.symbols(file.functions, file.variables) }}
{% endblock %}
